import traceback

from cpl_core.configuration.configuration_model_abc import ConfigurationModelABC
from cpl_core.console import Console, ForegroundColorEnum
from cpl_query.extension import List

from bot_core.configuration.file_logging_settings import FileLoggingSettings


class BotLoggingSettings(ConfigurationModelABC):
    def __init__(self):
        ConfigurationModelABC.__init__(self)
        self._files: List[FileLoggingSettings] = List(FileLoggingSettings)

    @property
    def files(self) -> List[FileLoggingSettings]:
        return self._files

    def from_dict(self, settings: dict):
        try:
            files = List(FileLoggingSettings)
            for s in settings:
                st = FileLoggingSettings()
                settings[s]["Key"] = s
                st.from_dict(settings[s])
                files.append(st)
            self._files = files
        except Exception as e:
            Console.set_foreground_color(ForegroundColorEnum.red)
            Console.write_line(f"[ ERROR ] [ {__name__} ]: Reading error in {type(self).__name__} settings")
            Console.write_line(f"[ EXCEPTION ] [ {__name__} ]: {e} -> {traceback.format_exc()}")
            Console.set_foreground_color(ForegroundColorEnum.default)
