from datetime import datetime

from cpl_core.database import TableABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC

from bot_data.model.server import Server


class Client(TableABC):
    def __init__(
        self,
        dc_id: int,
        smc: int,
        rmc: int,
        dmc: int,
        rcc: int,
        muc: int,
        server: Server,
        created_at: datetime = None,
        modified_at: datetime = None,
        id=0,
    ):
        self._client_id = id
        self._discord_client_id = dc_id
        self._sent_message_count = smc
        self._received_message_count = rmc
        self._deleted_message_count = dmc
        self._received_command_count = rcc
        self._moved_users_count = muc
        self._server: Server = server

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._client_id

    @property
    def discord_id(self) -> int:
        return self._discord_client_id

    @property
    @ServiceProviderABC.inject
    def name(self, bot: DiscordBotServiceABC) -> str:
        return bot.user.name

    @property
    def sent_message_count(self) -> int:
        return self._sent_message_count

    @sent_message_count.setter
    def sent_message_count(self, value: int):
        self._modified_at = datetime.now().isoformat()
        self._sent_message_count = value

    @property
    def received_message_count(self) -> int:
        return self._received_message_count

    @received_message_count.setter
    def received_message_count(self, value: int):
        self._modified_at = datetime.now().isoformat()
        self._received_message_count = value

    @property
    def deleted_message_count(self) -> int:
        return self._deleted_message_count

    @deleted_message_count.setter
    def deleted_message_count(self, value: int):
        self._modified_at = datetime.now().isoformat()
        self._deleted_message_count = value

    @property
    def received_command_count(self) -> int:
        return self._received_command_count

    @received_command_count.setter
    def received_command_count(self, value: int):
        self._modified_at = datetime.now().isoformat()
        self._received_command_count = value

    @property
    def moved_users_count(self) -> int:
        return self._moved_users_count

    @moved_users_count.setter
    def moved_users_count(self, value: int):
        self._modified_at = datetime.now().isoformat()
        self._moved_users_count = value

    @property
    def server(self) -> Server:
        return self._server

    @staticmethod
    def get_select_all_string() -> str:
        return str(
            f"""
            SELECT * FROM `Clients`;
        """
        )

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `Clients`
            WHERE `ClientId` = {id};
        """
        )

    @staticmethod
    def get_select_by_discord_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `Clients`
            WHERE `DiscordClientId` = {id};
        """
        )

    @staticmethod
    def get_select_by_server_id_string(id: int) -> str:
        return str(
            f"""
            SELECT * FROM `Clients`
            WHERE `ServerId` = {id};
        """
        )

    @staticmethod
    def get_select_by_discord_id_and_server_id_string(id: int, server_id: int) -> str:
        return str(
            f"""
            SELECT * FROM `Clients`
            WHERE `DiscordClientId` = {id}
            AND `ServerId` = {server_id};
        """
        )

    @property
    def insert_string(self) -> str:
        return str(
            f"""
            INSERT INTO `Clients` (
                `DiscordClientId`,
                `SentMessageCount`,
                `ReceivedMessageCount`,
                `DeletedMessageCount`,
                `ReceivedCommandsCount`,
                `MovedUsersCount`,
                `ServerId`,
                `CreatedAt`,
                `LastModifiedAt`
            ) VALUES (
                {self._discord_client_id},
                {self._sent_message_count},
                {self._received_message_count},
                {self._deleted_message_count},
                {self._received_message_count},
                {self._moved_users_count},
                {self._server.id},
                '{self._created_at}',
                '{self._modified_at}'
            );
        """
        )

    @property
    def udpate_string(self) -> str:
        return str(
            f"""
            UPDATE `Clients`
            SET `SentMessageCount` = {self._sent_message_count},
            `ReceivedMessageCount` = {self._received_message_count},
            `DeletedMessageCount` = {self._deleted_message_count},
            `ReceivedCommandsCount` = {self._received_command_count},
            `MovedUsersCount` = {self._moved_users_count},
            `LastModifiedAt` = '{self._modified_at}'
            WHERE `ClientId` = {self._client_id};
        """
        )

    @property
    def delete_string(self) -> str:
        return str(
            f"""
            DELETE FROM `Clients`
            WHERE `ClientId` = {self._client_id};
        """
        )
