from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.user_joined_game_server_repository_abc import (
    UserJoinedGameServerRepositoryABC,
)
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.model.user_joined_game_server import UserJoinedGameServer


class UserJoinedGameServerRepositoryService(UserJoinedGameServerRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        users: UserRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._users = users

        UserJoinedGameServerRepositoryABC.__init__(self)

    def get_user_joined_game_servers(self) -> List[UserJoinedGameServer]:
        joins = List(UserJoinedGameServer)
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.get_select_all_string()}",
        )
        results = self._context.select(UserJoinedGameServer.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get user-joined-game-server with id {result[0]}")
            joins.append(
                UserJoinedGameServer(
                    self._users.get_user_by_id(result[1]),
                    result[2],
                    result[3],
                    result[4],
                    result[5],
                    id=result[0],
                )
            )

        return joins

    def get_user_joined_game_server_by_id(self, id: int) -> UserJoinedGameServer:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.get_select_by_id_string(id)}",
        )
        result = self._context.select(UserJoinedGameServer.get_select_by_id_string(id))[0]
        return UserJoinedGameServer(
            self._users.get_user_by_id(result[1]),
            result[2],
            result[3],
            result[4],
            result[5],
            id=result[0],
        )

    def get_user_joined_game_servers_by_user_id(self, user_id: int) -> List[UserJoinedGameServer]:
        joins = List(UserJoinedGameServer)
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.get_select_by_user_id_string(user_id)}",
        )
        results = self._context.select(UserJoinedGameServer.get_select_by_user_id_string(user_id))
        for result in results:
            joins.append(
                UserJoinedGameServer(
                    self._users.get_user_by_id(result[1]),
                    result[2],
                    result[3],
                    result[4],
                    result[5],
                    id=result[0],
                )
            )

        return joins

    def get_active_user_joined_game_server_by_user_id(self, user_id: int) -> UserJoinedGameServer:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.get_select_by_user_id_string(user_id)}",
        )
        result = self._context.select(UserJoinedGameServer.get_select_active_by_user_id_string(user_id))[0]
        return UserJoinedGameServer(
            self._users.get_user_by_id(result[1]),
            result[2],
            result[3],
            result[4],
            result[5],
            id=result[0],
        )

    def find_active_user_joined_game_server_by_user_id(self, user_id: int) -> Optional[UserJoinedGameServer]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.get_select_by_user_id_string(user_id)}",
        )
        result = self._context.select(UserJoinedGameServer.get_select_active_by_user_id_string(user_id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return UserJoinedGameServer(
            self._users.get_user_by_id(result[1]),
            result[2],
            result[3],
            result[4],
            result[5],
            id=result[0],
        )

    def find_active_user_joined_game_servers_by_user_id(self, user_id: int) -> List[Optional[UserJoinedGameServer]]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.get_select_active_by_user_id_string(user_id)}",
        )
        result = List(UserJoinedGameServer)
        db_results = self._context.select(UserJoinedGameServer.get_select_active_by_user_id_string(user_id))

        for db_result in db_results:
            result.append(
                UserJoinedGameServer(
                    self._users.get_user_by_id(db_result[1]),
                    db_result[2],
                    db_result[3],
                    db_result[4],
                    db_result[5],
                    id=db_result[0],
                )
            )

        return result

    def add_user_joined_game_server(self, user_joined_game_server: UserJoinedGameServer):
        self._logger.trace(__name__, f"Send SQL command: {user_joined_game_server.insert_string}")
        self._context.cursor.execute(user_joined_game_server.insert_string)

    def update_user_joined_game_server(self, user_joined_game_server: UserJoinedGameServer):
        self._logger.trace(__name__, f"Send SQL command: {user_joined_game_server.udpate_string}")
        self._context.cursor.execute(user_joined_game_server.udpate_string)

    def delete_user_joined_game_server(self, user_joined_game_server: UserJoinedGameServer):
        self._logger.trace(__name__, f"Send SQL command: {user_joined_game_server.delete_string}")
        self._context.cursor.execute(user_joined_game_server.delete_string)

    def delete_user_joined_game_server_by_user_id(self, user_id: int):
        self._logger.trace(
            __name__,
            f"Send SQL command: {UserJoinedGameServer.delete_by_user_id_string}",
        )
        self._context.cursor.execute(UserJoinedGameServer.delete_by_user_id_string(user_id))
