import { Component, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { Queries } from "../../../../models/graphql/queries.model";
import { UserListQuery } from "../../../../models/graphql/query.model";
import { SpinnerService } from "../../../../services/spinner/spinner.service";
import { DataService } from "../../../../services/data/data.service";
import { User } from "../../../../models/data/user.model";
import { UserDTO } from "../../../../models/auth/auth-user.dto";
import { AuthService } from "src/app/services/auth/auth.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { TranslateService } from "@ngx-translate/core";

@Component({
  selector: "app-profile",
  templateUrl: "./profile.component.html",
  styleUrls: ["./profile.component.scss"]
})
export class ProfileComponent implements OnInit {

  user: User = { createdAt: "", modifiedAt: "" };

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private spinner: SpinnerService,
    private data: DataService,
    private auth: AuthService,
    private toast: ToastService,
    private translate: TranslateService
  ) {
  }

  ngOnInit() {
    this.data.getServerFromRoute(this.route).then(async (server) => {
      if (!this.route.snapshot.params["memberId"] || this.route.snapshot.params["memberId"] == "undefined") {
        this.router.navigate([`/server/${server.id}`]);
        return;
      }

      let authUser = await this.auth.getLoggedInUser();
      this.spinner.showSpinner();
      let user: UserDTO | null = authUser?.users?.find(u => u.server == server.id) ?? null;
      if (!user || user?.id != this.route.snapshot.params["memberId"] && !user?.isModerator) {
        this.toast.error(this.translate.instant("view.server.profile.permission_denied"), this.translate.instant("view.server.profile.permission_denied_d"));
        this.spinner.hideSpinner();
        this.router.navigate(["/server", server.id]);
        return;
      }

      this.data.query<User>(Queries.singleUserQuery, {
          filter: {
            id: this.route.snapshot.params["memberId"]
          }
        },
        function(data: UserListQuery) {
          return data.users.length > 0 ? data.users[0] : null;
        }
      ).subscribe(user => {
        this.user = user;
        this.spinner.hideSpinner();
      });
    });


  }
}
